# Base for scenario views

from analyses.analysisbase import forms as app_forms
from analyses.analysisbase import tables as app_tables
from analyses.analysisbase.views import helpers
from analyses.main import analyses_navigationlist

from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _

from installations import models as installations_models
from installations.views.main import installations_navigationlist

from utils import crud
from utils import forms as utils_forms
from utils import permissions
from utils.autocomplete import autocomplete
from utils.trees import models as trees_models
from utils.trees import views as trees_views
from utils.versioning import versionedcrud
from utils.versioning.session import get_current_version
from utils.versioning.models import WorkVersion


class MixIn(object):
    "Common items for object views"
    include_attachments = True
    include_links = True

##    appconfig = app_models.AppConfig
##    model = appconfig.get_model('scenario')

    basetabs = [
        {'name': 'scenario',
            'label': _('Description')},
        {'name': 'causetree',
            'label': _('Cause tree')},
    ]
    def setup_tabs(self):
        self.tabs = self.basetabs[:]
        if self.object and not self.object.suggestionlist and self.basetabs:
            self.tabs.extend([
                {'name': 'evaluation',
                    'label': _('Evaluation')},
            ])

    def add_to_extra_context(self, extra_context, object):
        if not object.suggestionlist:
            extra_context['navigationlist'] =  installations_navigationlist()
        extra_context['currentanalysistype'] = self.appconfig.analysistype_short
        if object.scope:
            extra_context['breadcrumbs'] = object.scope.get_breadcrumbs()
        if object.suggestionlist:
            extra_context['breadcrumbs'] = [
                {'url': object.suggestionlist.get_absolute_url(),
                'label': u'%s' % object.suggestionlist}]

class display_base(MixIn, versionedcrud.DisplayObjectView):
    allow_move = False
    allow_publish = False
    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn.add_to_extra_context(self, extra_context, object)
        if self.allow_move:
            if object.suggestionlist:
                if object.may_change(self.request.user):
                    extra_context['defaultbuttons'].append({'url': object.get_move_to_sl_url(), 'label': _('Move to SL')})
                if object.may_copytosl(self.request.user):
                    extra_context['defaultbuttons'].append({'url': object.get_copy_to_sl_url(), 'label': _('Copy to SL')})
            else:
                if object.may_change(self.request.user):
                    extra_context['defaultbuttons'].append({'url': object.get_move_url(), 'label': _('Move')})
                    extra_context['defaultbuttons'].append({'url': object.get_copy_url(), 'label': _('Copy')})
                if object.may_copytosl(self.request.user):
                    extra_context['defaultbuttons'].append({'url': object.get_copy_to_sl_url(), 'label': _('Copy to SL')})
        if self.allow_publish and object.may_publish(self.request.user):
            extra_context['defaultbuttons'].append({'url': object.get_publish_url(), 'label': _('Publish')})

class display_scenario(display_base):
    form_class = None
    allow_move = True
    allow_publish = True

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.ScenarioSheet, model=self.model)

class display_causetree(display_base):
    allow_delete = False
    form_class = None
    template = 'analysisbase/scenario_causetree.html'

    def add_to_extra_context(self, extra_context, object):
        super(display_causetree, self).add_to_extra_context(extra_context, object)
        treeobjects = self.appconfig.get_model('treeelement').tree.filter(scenario=object
            ).get_versionedobjects(version=self.version).select_related_versioned()
        extra_context['treeobjects'] = treeobjects

class display_evaluation(display_base):
    allow_delete = False
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.ScenarioEvaluationSheet, model=self.model)

##class display(MixIn, crud.TabContainer):
##    tabviews = {
##        'scenario': display_scenario(),
##        'causetree': display_causetree(),
##        'evaluation': display_evaluation(),
##    }
##    default = 'scenario'

class edit_scenario(MixIn, versionedcrud.EditObjectView):
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.ScenarioForm, model=self.model)

class edit_causetree(MixIn, trees_views.edit_tree):
    template = 'analysisbase/scenario_causetree_edit.html'

    def __new__(cls, *args, **kwargs):
        cls.treeelement_class = cls.appconfig.get_model('treeelement')
        return super(edit_causetree, cls).__new__(cls, *args, **kwargs)

    def create_element(self, parent, type, name, treeelementkwargs={}):
        if type == 'cause':
            instance = self.appconfig.get_model('event')(name=name)
            return instance.save(self.request.user, parent=parent, treeelementkwargs={'scenario': self.object})
        if type == 'protectionlayer':
            instance = self.appconfig.get_model('protectionlayer')(name=name, SL=bool(self.object.suggestionlist))
            return instance.save(self.request.user, parent=parent, treeelementkwargs={'scenario': self.object})
        return super(edit_causetree, self).create_element(parent=parent, type=type,
            name=name, treeelementkwargs={'scenario': self.object} )

    def get_treeobjects(self, object):
        return self.appconfig.get_model('treeelement').tree.filter(scenario=object)

    def add_to_extra_context(self, extra_context, object):
        trees_views.edit_tree.add_to_extra_context(self,extra_context, object)
        MixIn.add_to_extra_context(self, extra_context, object)

class edit_evaluation(MixIn, versionedcrud.EditObjectView):
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.ScenarioEvaluationForm, model=self.model)

##class edit(MixIn, crud.TabContainer):
##    tabviews = {
##        'scenario': edit_scenario(),
##        'causetree': edit_causetree(),
##        'evaluation': edit_evaluation(),
##    }
##    default = 'scenario'

class delete(MixIn, versionedcrud.DeleteObjectView):
    basetabs = []
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.ScenarioSheet, model=self.model)

    def get_default_url(self, object):
        if object.suggestionlist:
            return object.suggestionlist.get_absolute_url()
        return object.scope.get_redirect_url()

class report(MixIn, versionedcrud.ReportObjectView):
    form_class = utils_forms.ReportForm
    def add_to_extra_context(self, extra_context, object):
        versionedcrud.ReportObjectView.add_to_extra_context(self,extra_context, object)
        MixIn.add_to_extra_context(self, extra_context, object)

class add_to_scope(MixIn, versionedcrud.AddObjectView):
    basetabs = []
    parent_class = installations_models.Scope
    parent_field = 'scope'
    parent_tab = 'scenarios'
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.ScenarioAddForm, model=self.model)

    def check_permission(self, object):
        # override because scope is no a real parent permissionwise
        if not self.model.may_add(self.request.user):
            return permissions.redirect_to_not_allowed()

    def save_form(self, form):
        object = super(add_to_scope, self).save_form(form)
        parent = None
        for item in ('consequence',):
            data = form.cleaned_data.get(item)
            if data:
                instance = self.appconfig.get_model('event')(name=data)
                parent = instance.save(self.request.user, parent=parent, treeelementkwargs={'scenario':object})
        return object

    def get_redirect(self, refform, object):
        return HttpResponseRedirect(object.get_absolute_url('causetree'))

class add_to_suggestionlist(add_to_scope):
    def __new__(cls, *args, **kwargs):
        cls.parent_class = cls.appconfig.get_model('suggestionlist')
        cls.parent_field = 'suggestionlist'
        return super(add_to_suggestionlist, cls).__new__(cls, *args, **kwargs)

class CompareMixIn(MixIn):
    def setup_tabs(self):
        self.tabs = [
            {'name': 'scenario',
                'label': _('Scenario')},
            {'name': 'causetree',
                'label': _('Cause tree')},
        ]
    def get_tab_url(self, object, tab):
        return object.get_compare_url(self.version2,tab )

class compare_scenario(CompareMixIn, versionedcrud.CompareObjectView):
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.ScenarioCompareSheet, model=self.model)
        super(compare_scenario, self).setup_from_args(*args, **kwargs)

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.CompareObjectView.add_to_extra_context(self,extra_context, object)
        MixIn.add_to_extra_context(self, extra_context, object)

class compare_causetree(CompareMixIn, versionedcrud.CompareObjectView):
    form_class = None
    template = 'analysisbase/scenario_compare_causetree.html'

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.CompareObjectView.add_to_extra_context(self,extra_context, object)
        MixIn.add_to_extra_context(self, extra_context, object)
        # Exclude the navigationlist because there is no room
        extra_context['navigationlist'] = None
        extra_context['currentanalysistype'] = None
        treeobjects = self.appconfig.get_model('treeelement').tree.filter(scenario = object
            ).get_versionedobjects(version=self.object.version or WorkVersion).select_related_versioned()
        extra_context['treeobjects'] = treeobjects
        treeobjects2 = self.appconfig.get_model('treeelement').tree.filter(scenario = self.object2
            ).get_versionedobjects(version=self.object2.version or WorkVersion).select_related_versioned()
        extra_context['treeobjects2'] = treeobjects2
        extra_context['version2'] = self.object2.version or WorkVersion


##class compare(MixIn, crud.TabContainer):
##    tabviews = {
##        'scenario': compare_scenario(),
##        'causetree': compare_causetree(),
##    }
##    default = 'scenario'

class move(MixIn, versionedcrud.EditObjectView):
    form_class = None
    basetabs = []

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = app_forms.ScenarioMoveForm
        super(move, self).setup_from_args(*args, **kwargs)

    # crud overrides
    def get_form(self, object, **formargs):
        # do not pass in instance, since we are not using a modelform
        if self.request.method == 'POST':
            form = self.form_class(data=self.request.POST, files=self.request.FILES)
        else:
            form = self.form_class()
        if self.model.may_add(self.request.user):
            form.fields['target'].queryset = installations_models.Scope.objects.all().get_versionedobjects(version=WorkVersion)
        self.setup_form(form, object)
        return form

    def save_form(self, form):
        target = form.cleaned_data['target']
        if target:
            self.object.move(user=self.request.user, scope = target)
        return self.object

class move_to_sl(MixIn, versionedcrud.EditObjectView):
    form_class = None
    basetabs = []

    def check_permission(self, object):
        # permission to edit this object
        c = super(move_to_sl, self).check_permission(object)
        if c:
            return c
        # permission to edit a suggestionlist
        c = permissions.check_permission(self.request.user, self.appconfig.get_model('suggestionlist').editpermission)
        if c:
            return c

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = app_forms.ScenarioMoveForm
        super(move_to_sl, self).setup_from_args(*args, **kwargs)

    # crud overrides
    def get_form(self, object, **formargs):
        # do not pass in instance, since we are not using a modelform
        if self.request.method == 'POST':
            form = self.form_class(data=self.request.POST, files=self.request.FILES)
        else:
            form = self.form_class()
        form.fields['target'].queryset = self.appconfig.get_model('suggestionlist').objects.filter(system=False).get_versionedobjects(version=WorkVersion)
        self.setup_form(form, object)
        return form

    def save_form(self, form):
        target = form.cleaned_data['target']
        if target:
            self.object.move(user=self.request.user, suggestionlist = target)
        return self.object

class copy(move):
    def save_form(self, form):
        target = form.cleaned_data['target']
        if target:
            self.newobject = self.object.copy(user=self.request.user, scope=target, suffix='[copy]')
        return self.object

    def get_redirect(self, refform, object):
        return HttpResponseRedirect(self.newobject.get_absolute_url('scenario'))

class copy_to_sl(move_to_sl):
    def check_permission(self, object):
        # permission to edit a suggestionlist
        c = permissions.check_permission(self.request.user, self.appconfig.get_model('suggestionlist').editpermission)
        if c:
            return c
    def save_form(self, form):
        target = form.cleaned_data['target']
        if target:
            self.newobject = self.object.copy(user=self.request.user, suggestionlist=target, suffix='[copy]')
        return self.object

    def get_redirect(self, refform, object):
        return HttpResponseRedirect(self.newobject.get_absolute_url('scenario'))

class publish(MixIn, versionedcrud.PublishObjectView):
    basetabs = []

def type_lookup(request, appconfig):
    model = appconfig.get_model('scenario')
    queryset = model.objects.filter(suggestionlist__isnull=True).values('type').order_by('type').distinct()
    return autocomplete(request, queryset, 'type', 'type')

def name_lookup(request, appconfig):
    model = appconfig.get_model('scenario')
    (version, special) = get_current_version(request)
    queryset = model.objects.filter(suggestionlist__isnull=True).get_versionedobjects(version=version).values('name').order_by('name')
    return autocomplete(request, queryset, 'name', 'name')

class ScenarioFilter(versionedcrud.Filter):
    form_class = None
    filters = [
                {'filteritem': 'name', 'parameter': 'name__icontains', 'type': 'normal'},
                {'filteritem': 'type', 'parameter': 'type', 'type': 'normal'},
                {'filteritem': 'is_ok', 'parameter': 'is_ok', 'type': 'normal'},
            ]
    def get_form(self):
        if not self.form_class:
            return utils_forms.create_class_with_meta(app_forms.ScenarioFilterForm, model=self.model)
        return self.form_class

class ScenarioListObject(helpers.TableListObject):
    defaultsort = 'name'
    default_table_class = app_tables.ScenariosTable

    def get_title(self):
        return u'%s %s' % (_('Scenarios'), self.appconfig.analysistype)

    def get_query_set(self):
        return self.model.objects.filter(suggestionlist__isnull=True).get_versionedobjects(self.version).select_related_versioned('scope')

    def add_html(self, html_strings, queryset, user, language, choices):
        if choices.get('scenarios'):
            for item in queryset:
                html_strings.append('<pdf:nextpage>')
                html_strings.extend(item.get_report_body(user, language, choices))

    def add_css(self, css_files, queryset, user, language, choices):
        if choices.get('scenarios'):
            if queryset:
                css_files.extend(queryset[0].get_report_css(language, choices))

class scenarios(helpers.TableListView):
    "List of scenarios"
    default_table_class = app_tables.ScenariosTable

    filter = ScenarioFilter()
    defaultsort = 'name'
    paginate_by = 100
    allow_pdf = True
    pdfsendmail = True

    def get_title(self):
        return u'%s %s' % (_('Scenarios'), self.appconfig.analysistype)

    def get_query_set(self, request):
        return self.model.objects.filter(suggestionlist__isnull=True).get_versionedobjects(self.version)

    def add_to_extra_context(self, extra_context):
        super(scenarios, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] =  analyses_navigationlist(self.appconfig.namespace)
        extra_context['tablist'] =  self.get_navigationlist(self.appconfig.namespace)
        extra_context['currenttab'] = 'scenarios'
        extra_context['breadcrumbs'] = [{'label': _('Lists'), 'url': u''}]

    def get_navigationlist(self, namespace):
        return localnavigationlist(namespace)

class activate(object):
    def __call__(self, request, object_id=None):
        request.session['analysistype'] = self.appconfig.namespace
        if request.META.get('HTTP_REFERER', None):
            return HttpResponseRedirect(request.META['HTTP_REFERER'])
        return HttpResponseRedirect('/')

def localnavigationlist(namespace):
    return [
            {   'name': u'scenarios',
                'url': reverse('%s_scenarios' % namespace),
                'label': _('Scenarios')},
            {   'name': u'protectionlayers',
                'url': reverse('%s_protectionlayers' % namespace),
                'label': _('Protection layers')},
            {   'name': u'measures',
                'url': reverse('%s_measures' % namespace),
                'label': _('Measures')},
            {   'name': u'attentionpoints',
                'url': reverse('%s_attentionpoints' % namespace),
                'label': _('Attention points')},
        ]

