from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from utils import tables
from utils import fields as utils_fields
import tasks.models as app_models

class TasksTable(tables.ModelTable):
    content_object = tables.Column(
            verbose_name = _('Related object'),
            data = lambda x: x.data.content_object,
            link = lambda x: x.data.content_object.get_absolute_url(),
        )
    task = tables.Column(
            verbose_name = _('Task'),
            link = lambda x: x.data.get_absolute_url(),
        )
    user = tables.Column(
            verbose_name = _('Responsable'),
            data = lambda x: x.data.user and utils_fields.friendly_username(x.data.user)
        )
    duedate = tables.DateColumn(
            verbose_name = _('Duedate'),
            format = settings.DATE_OUTPUT_FORMAT
        )
    completiondate = tables.DateColumn(
            verbose_name = _('Compl. date'),
            format = settings.DATE_OUTPUT_FORMAT
        )

    class Meta:
        model = app_models.Task
        columns = ('content_object', 'task', 'user', 'duedate', 'completiondate')


class TasksCSVTable(tables.ModelTable):
    content_object = tables.Column(
            verbose_name = _('Related object'),
            data = lambda x: x.data.content_object,
        )
    user = tables.Column(
            verbose_name = _('Responsable'),
            data = lambda x: x.data.user and utils_fields.friendly_username(x.data.user)
        )
    duedate = tables.DateColumn(
            verbose_name = _('Duedate'),
            format = settings.DATE_OUTPUT_FORMAT
        )
    completiondate = tables.DateColumn(
            verbose_name = _('Compl. date'),
            format = settings.DATE_OUTPUT_FORMAT
        )

    class Meta:
        model = app_models.Task
        columns = ('content_object', 'task', 'user', 'duedate', 'completiondate', 'description', 'result')
