from django.utils import simplejson

from django.http import HttpResponse

def autocomplete(request, queryset, displayfield, usefield, default_limit=15):
    """
    View used to return items into the jquery autocomplete plugin
    """
    q = request.GET.get('term')
    if not q:
        return HttpResponse(mimetype='text/plain')
    limit = request.GET.get('limit', default_limit)
    try:
        limit = int(limit)
    except ValueError:
        limit = default_limit
    filter = {'%s__icontains' % displayfield : q}
    data = queryset.filter(**filter)[:limit]
    results = []
    for item in data:
        if isinstance(item, dict):
            results.append({'label': item.get(displayfield), 'value': item.get(usefield)})
        else:
            results.append({'label': getattr(item,displayfield), 'value': getattr(item,usefield)})
    json = simplejson.dumps(results)
    return HttpResponse(json, mimetype='application/json')
