"""
Search app
"""
from django.db.models import Q
from utils.versioning.models import VersionedModel

class Search(object):
    permission = None
    fields = []
    model = None
    name = None
    select_related_versioned = False

    def search(self, query, user, version, max):
        results = []
        if self.model:
            if self.permission and user.has_perm(self.permission):
                query_items = query.split(' ')
                filters = []
                for q in query_items:
                    qf = []
                    for field in self.fields:
                        qf.append(Q(**{'%s__icontains' % field: q}))
                    filters.append(reduce(lambda x, y: x|y, qf))
                filter = reduce(lambda x, y: x&y, filters)
                items = self.get_queryset(filter, version, max)
                for item in items:
                    results.append(
                            {
                                'objecttype': u'%s' % self.get_objecttype(),
                                'label': self.get_label(item),
                                'url': self.get_url(item)
                            }
                        )
        return results

    def get_queryset(self, filter, version, max):
        qs = self.model.objects.filter(filter)
        if issubclass(self.model, VersionedModel):
            if self.select_related_versioned:
                qs = qs.get_versionedobjects(version).select_related_versioned()
            else:
                qs = qs.get_versionedobjects(version)
        if max:
            qs = qs[:max]
        return qs

    def get_objecttype(self):
        if self.name:
            return self.name
        if self.model:
            return self.model._meta.verbose_name
        return u''

    def get_label(self, object):
        return u'%s' % object

    def get_url(self, object):
        return object.get_absolute_url()


class Searches(object):
    def __init__(self):
        self._registry = []

    def register(self, searchclass):
        """
        Registers the given search subclass.
        """
        if searchclass in self._registry:
            # do nothing if already registered
            pass
        else:
            self._registry.append(searchclass)

    def get_searches(self, objecttype = None):
        if objecttype:
            return [i for i in self._registry if objecttype == u'%s' % i().get_objecttype()]
        return self._registry

    def get_types(self):
        items = list(set([u'%s' % i().get_objecttype() for i in self._registry]))
        items.sort()
        return zip(items,items)

searches = Searches()